//+------------------------------------------------------------------+
//|                                                    Functions.mqh |
//|                                  2019 - 2021, dimitri pecheritsa |
//|                             https://www.mql5.com/en/users/dmipec |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| functions used in the heap sort example script                   |
//|  symbols load (destination symbols)                              |
//|  symbols keys color (source symbols, destination keys)           |
//|  symbols print (symbols)                                         |
//|  array sort (keys, items, algorithm, order)                      |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| symbols load                                                     |
//|  load symbols from the terminal to the destination array         |
//+------------------------------------------------------------------+
int SymbolsLoad(string& dst_symbols[])
  {
   int total=SymbolsTotal(false);
   for(int i=0; i<total; i++)
     {
      int size=ArraySize(dst_symbols);
      ArrayResize(dst_symbols,size+1,size*2);
      dst_symbols[size]=SymbolName(i,false);
     }
   return total;
  }
//+------------------------------------------------------------------+
//| symbols keys color                                               |
//|  fill the destination keys array by extracting symbol color from |
//|the source symbols array for each of its element                  |
//+------------------------------------------------------------------+
void SymbolKeysColor(string& symbols[],color& keys[])
  {
   int total=ArraySize(symbols);
   ArrayResize(keys,total);
   for(int i=0; i<total; i++)
      keys[i]=(color)SymbolInfoInteger(symbols[i],SYMBOL_BACKGROUND_COLOR);
  }
//+------------------------------------------------------------------+
//| symbols print                                                    |
//|  print a table of symbols consiting of three columns: name,      |
//|color, and digits                                                 |
//+------------------------------------------------------------------+
void SymbolsPrint(string& symbols[])
  {
//--- create horizontal table line
   string line;
   StringInit(line,68,'-');
//--- header
   printf(line);
   printf("%13s | %34s | %8s","symbol","color name (id)","digits");
   printf(line);
//--- body
   int total=ArraySize(symbols);
   for(int i=0; i<total; i++)
     {
      //--- some symbol data
      int clr_id=(int)SymbolInfoInteger(symbols[i],SYMBOL_BACKGROUND_COLOR);
      string clr=StringFormat("%s (%d)",ColorToString((color)clr_id,true),clr_id);
      int digits=(int)SymbolInfoInteger(symbols[i],SYMBOL_DIGITS);
      //--- out
      printf("%13s | %34s | %8d",symbols[i],clr,digits);
     }
  }
//+------------------------------------------------------------------+
//| array sort                                                       |
//|  a universal function which sorts the items array by the keys    |
//|array with the specified concrete sorting algorithm, in accending |
//|or descending order. any compatible sorting algorithm can be used |
//|as the function parameter. the items array is ignored if its      |
//|length does not match the keys length                             |
//+------------------------------------------------------------------+
#include <Mqh\Algorithms\ASorter.mqh>
template<typename TKey,typename TItem>
void ArraySort(TKey& keys[],TItem& items[],ASorter<TKey,TItem>* algorithm,bool accending_order=true)
  {
//--- prepare algorithm
   ArrayCopy(algorithm.keys,keys);
   ArrayCopy(algorithm.items,items);
//--- sort keys and items
   algorithm.Sort();
//--- save sorted items
   ArrayCopy(keys,algorithm.keys);
   ArrayCopy(items,algorithm.items);
//--- keep order
   if(!accending_order)
     {
      ArrayReverse(keys);
      ArrayReverse(items);
     }
//--- clear
   if(CheckPointer(algorithm)==POINTER_DYNAMIC)
      delete algorithm;
  }
//+------------------------------------------------------------------+
